#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OLE Compound Files (OLECF) parser."""

import unittest

from plaso.containers import warnings
from plaso.parsers import olecf
from plaso.parsers import olecf_plugins  # pylint: disable=unused-import

from tests.parsers import test_lib


class OLECFParserTest(test_lib.ParserTestCase):
  """Tests for the OLE Compound Files (OLECF) parser."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = olecf.OLECFParser()

    number_of_plugins = len(parser._plugin_classes)

    parser.EnablePlugins([])
    self.assertEqual(len(parser._plugins_per_name), 0)

    parser.EnablePlugins(parser.ALL_PLUGINS)
    # Extract 1 for the default plugin.
    self.assertEqual(len(parser._plugins_per_name), number_of_plugins - 1)

    parser.EnablePlugins(['olecf_document_summary'])
    self.assertEqual(len(parser._plugins_per_name), 1)

  def testParse(self):
    """Tests the Parse function."""
    parser = olecf.OLECFParser()
    storage_writer = self._ParseFile(['Document.doc'], parser)

    # OLE Compound File information:
    #     Version             : 3.62
    #     Sector size         : 512
    #     Short sector size   : 64

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 12)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'olecf:item',
        'modification_time': '2013-05-16T02:29:49.7850000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8)
    self.CheckEventData(event_data, expected_event_values)

    storage_writer = self._CreateStorageWriter()
    parser_mediator = self._CreateParserMediator(storage_writer)

    parser = olecf.OLECFParser()
    parser.ParseFileObject(parser_mediator, None)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    generator = storage_writer.GetAttributeContainers(
        warnings.ExtractionWarning.CONTAINER_TYPE)

    test_warnings = list(generator)
    test_warning = test_warnings[0]
    self.assertIsNotNone(test_warning)

    expected_message = (
        'unable to open file with error: pyolecf_file_open_file_object: ')
    self.assertTrue(test_warning.message.startswith(expected_message))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows EventLog custom event formatter helpers."""

from plaso.formatters import interface
from plaso.formatters import logger
from plaso.formatters import manager


class WindowsEventLogMessageFormatterHelper(
    interface.CustomEventFormatterHelper):
  """Windows EventLog message formatter helper."""

  IDENTIFIER = 'windows_eventlog_message'

  def __init__(self):
    """Initialized a indows EventLog message formatter helper."""
    super(WindowsEventLogMessageFormatterHelper, self).__init__()
    self._winevt_resources_helper = None

  def FormatEventValues(self, output_mediator, event_values):
    """Formats event values using the helper.

    Args:
      output_mediator (OutputMediator): output mediator.
      event_values (dict[str, object]): event values.
    """
    if not self._winevt_resources_helper:
      self._winevt_resources_helper = output_mediator.GetWinevtResourcesHelper()

    message_string = None
    provider_identifier = event_values.get('provider_identifier', None)
    source_name = event_values.get('source_name', None)
    message_identifier = event_values.get('message_identifier', None)
    event_version = event_values.get('event_version', None)
    if (provider_identifier or source_name) and message_identifier:
      message_string_template = self._winevt_resources_helper.GetMessageString(
          provider_identifier, source_name, message_identifier, event_version)
      if message_string_template:
        string_values = [
            string or '' for string in event_values.get('strings', [])]

        try:
          message_string = message_string_template.format(*string_values)
        except (IndexError, TypeError) as exception:
          logger.error((
              'Unable to format message: 0x{0:08x} of provider: {1:s} '
              'template: "{2:s}" and strings: "{3:s}" with error: '
              '{4!s}').format(
                  message_identifier, provider_identifier or '',
                  message_string_template, ', '.join(string_values), exception))
          # Unable to create the message string.
          # TODO: consider returning the unformatted message string.

    event_values['message_string'] = message_string


manager.FormattersManager.RegisterEventFormatterHelper(
    WindowsEventLogMessageFormatterHelper)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the event source attribute containers."""

import unittest

from plaso.containers import event_sources

from tests import test_lib as shared_test_lib


class EventSourceTest(shared_test_lib.BaseTestCase):
  """Tests for the event source attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = event_sources.EventSource()

    expected_attribute_names = [
        'data_type', 'file_entry_type', 'path_spec']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


class FileEntryEventSourceTest(shared_test_lib.BaseTestCase):
  """Tests for the file entry event source attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = event_sources.FileEntryEventSource()

    expected_attribute_names = [
        'data_type', 'file_entry_type', 'path_spec']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Plist parser plugin for iPod, iPad and iPhone storage plist files."""

from plaso.containers import events
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class IPodPlistEventData(events.EventData):
  """iPod plist event data.

  Attributes:
    device_class (str): device class.
    device_identifier (str): identifier of the device.
    family_identifier (str): identifier of the device family.
    firmware_version (str): firmware version.
    imei (str): IMEI (International Mobile Equipment Identity).
    last_connected_time (dfdatetime.DateTimeValues): last date and time
        the iPod, iPad or iPhone storage (device) was connected.
    serial_number (str): serial number.
    use_count (str): number of times the device was used.
  """

  DATA_TYPE = 'ipod:device:entry'

  def __init__(self):
    """Initializes event data."""
    super(IPodPlistEventData, self).__init__(data_type=self.DATA_TYPE)
    self.device_class = None
    self.device_identifier = None
    self.family_identifier = None
    self.firmware_version = None
    self.imei = None
    self.last_connected_time = None
    self.serial_number = None
    self.use_count = None


class IPodPlugin(interface.PlistPlugin):
  """Plist parser plugin for iPod, iPad and iPhone storage plist files."""

  NAME = 'ipod_device'
  DATA_FORMAT = 'iPod, iPad and iPhone plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('com.apple.iPod.plist')])

  PLIST_KEYS = frozenset(['Devices'])

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, match=None, **unused_kwargs):
    """Extract device information from the iPod plist.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    devices = match.get('Devices', {})
    for device_identifier, plist_key in devices.items():
      event_data = IPodPlistEventData()
      event_data.device_identifier = device_identifier
      event_data.device_class = plist_key.get('Device Class', None)
      event_data.family_identifier = plist_key.get('Family ID', None)
      event_data.firmware_version = plist_key.get(
          'Firmware Version String', None)
      event_data.imei = plist_key.get('IMEI', None)
      event_data.last_connected_time = self._GetDateTimeValueFromPlistKey(
          plist_key, 'Connected')
      event_data.serial_number = plist_key.get('Serial Number', None)
      event_data.use_count = plist_key.get('Use Count', None)

      parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(IPodPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Script to extract information about the supported data formats."""

import argparse
import collections
import importlib
import inspect
import os
import pkgutil
import sys

from urllib import parse as urllib_parse

from dtfabric import reader as dtfabric_reader
from dtfabric import registry as dtfabric_registry

import plaso

from plaso.parsers import interface as parsers_interface
from plaso.parsers import plugins as parsers_plugins


class DataFormatDescriptor(object):
  """Descriptor of a specific data format.

  Attributes:
    category (str): category of the data format, for example "File formats" or
        "OLE Compound File formats".
    name (str): name of the data format, for example "Chrome Extension
        activity database".
    url (str): URL to more information about the data format.
  """

  def __init__(self, category=None, name=None, url=None):
    """Initializes a data format descriptor.

    Args:
      category (Optional[str]): category of the data format, for example
        "File formats" or "OLE Compound File formats".
      name (Optional[str]): name of the data format, for example "Chrome
        Extension activity database".
      url (Optional[str]): URL to more information about the data format.
    """
    super(DataFormatDescriptor, self).__init__()
    # TODO: add data format name aliases.
    self.category = category
    self.name = name
    self.url = url


class DataFormatInformationExtractor(object):
  """Data format information extractor."""

  _CATEGORIES_OUTPUT_ORDER = [
      'Storage media image file formats',
      'Volume system formats',
      'File system formats',
      'File formats',
      'Bencode file formats',
      'Browser cookie formats',
      'Compound ZIP file formats',
      'ESE database file formats',
      'JSON-L log file formats',
      'OLE Compound File formats',
      'Property list (plist) formats',
      'SQLite database file formats',
      'Text-based log file formats',
      'Windows Registry formats']

  # TODO: consider extending Plaso parsers and parser plugins with metadata that
  # contain this information.
  _DATA_FORMAT_CATEGORY_PER_PACKAGE_PATH = {
      'plaso/parsers': 'File formats',
      'plaso/parsers/bencode_plugins': 'Bencode file formats',
      'plaso/parsers/cookie_plugins': 'Browser cookie formats',
      'plaso/parsers/czip_plugins': 'Compound ZIP file formats',
      'plaso/parsers/esedb_plugins': 'ESE database file formats',
      'plaso/parsers/jsonl_plugins': 'JSON-L log file formats',
      'plaso/parsers/olecf_plugins': 'OLE Compound File formats',
      'plaso/parsers/plist_plugins': 'Property list (plist) formats',
      'plaso/parsers/sqlite_plugins': 'SQLite database file formats',
      'plaso/parsers/text_plugins': 'Text-based log file formats',
      'plaso/parsers/winreg_plugins': 'Windows Registry formats'}

  _DTFORMATS_URL_PREFIX = (
      'https://github.com/libyal/dtformats/blob/main/documentation')

  # Names of parsers and parser plugins to ignore.
  _PARSER_NAME_IGNORE_LIST = frozenset([
      'base_parser',
      'base_plugin',
      'bencode_plugin',
      'cookie_plugin',
      'czip_plugin',
      'esedb_plugin',
      'filestat',
      'jsonl_plugin',
      'mrulistex_shell_item_list',
      'mrulistex_string',
      'mrulistex_string_and_shell_item',
      'mrulistex_string_and_shell_item_list',
      'mrulist_shell_item_list',
      'olecf_default',
      'olecf_plugin',
      'plist_default',
      'plist_plugin',
      'sqlite_plugin',
      'text_plugin',
      'winreg_default',
      'winreg_plugin'])

  _STANDARD_TEXT_PER_CATEGORY = {
      'File system formats': (
          'File System Format support is provided by [dfVFS]'
          '(https://dfvfs.readthedocs.io/en/latest/sources/'
          'Supported-formats.html#file-systems).'),
      'Storage media image file formats': (
          'Storage media image file format support is provided by [dfVFS]'
          '(https://dfvfs.readthedocs.io/en/latest/sources/'
          'Supported-formats.html#storage-media-types).'),
      'Volume system formats': (
          'Volume system format support is provided by [dfVFS]'
          '(https://dfvfs.readthedocs.io/en/latest/sources/'
          'Supported-formats.html#volume-systems).')}

  def FormatDataFormats(self, data_format_descriptors):
    """Formats data format information.

    Args:
      data_format_descriptors (list[DataFormatDescriptor]): data format
          descriptors.

    Returns:
      str: information about data formats.
    """
    lines = [
        '## Supported Formats',
        '',
        'The information below is based of version {0:s}'.format(
            plaso.__version__),
        '']

    descriptors_per_category = collections.defaultdict(list)
    for data_format_descriptor in data_format_descriptors:
      descriptors_per_category[data_format_descriptor.category].append(
          data_format_descriptor)

    for category in self._CATEGORIES_OUTPUT_ORDER:
      lines.append('### {0:s}'.format(category))
      lines.append('')

      standard_text = self._STANDARD_TEXT_PER_CATEGORY.get(category, None)
      if standard_text is not None:
        lines.append(standard_text)

      lines_per_category = []
      data_format_descriptors = descriptors_per_category.get(category, [])
      for data_format_descriptor in sorted(
          data_format_descriptors, key=lambda cls: cls.name):
        url = data_format_descriptor.url

        # TODO: add support for more generic generation of using information.
        if url.startswith('dtformats:'):
          url = url.split(':')[1]
          url = urllib_parse.quote(url)
          url = '{0:s}/{1:s}.asciidoc'.format(self._DTFORMATS_URL_PREFIX, url)
          line = '* [{0:s}]({1:s})'.format(data_format_descriptor.name, url)

        elif url.startswith('libyal:'):
          library_name, url = url.split(':')[1:3]
          library_url = 'https://github.com/libyal/{0:s}'.format(library_name)
          url = urllib_parse.quote(url)
          url = '{0:s}/blob/main/documentation/{1:s}.asciidoc'.format(
              library_url, url)
          line = '* [{0:s}]({1:s}) using [{2:s}]({3:s})'.format(
              data_format_descriptor.name, url, library_name, library_url)

        elif url.startswith('http:') or url.startswith('https:'):
          line = '* [{0:s}]({1:s})'.format(data_format_descriptor.name, url)

        else:
          line = '* {0:s}'.format(data_format_descriptor.name)

        lines_per_category.append(line)

      # Sort the lines per category case-insensitive and ignoring
      # non-alphanumeric characters.
      lines_per_category = sorted(
          lines_per_category,
          key=lambda name: ''.join(filter(str.isalnum, name.lower())))
      lines.extend(lines_per_category)

      if standard_text or data_format_descriptors:
        lines.append('')

    return '\n'.join(lines)

  def _GetDataFormatInformationFromPackage(self, package):
    """Retrieves event data attribute containers from a package.

    Args:
      package (list[str]): package name segments such as ["plaso", "parsers"].

    Returns:
      list[DataFormatDescriptor]: data format descriptors.
    """
    data_format_descriptors = []
    package_path = '/'.join(package)
    for _, name, is_package in pkgutil.iter_modules(path=[package_path]):
      sub_package = list(package)
      sub_package.append(name)
      if is_package:
        sub_data_format_descriptors = (
            self._GetDataFormatInformationFromPackage(sub_package))
        data_format_descriptors.extend(sub_data_format_descriptors)
      else:
        module_path = '.'.join(sub_package)
        try:
          module_object = importlib.import_module(module_path)
        except ImportError:
          module_object = None

        for _, cls in inspect.getmembers(module_object, inspect.isclass):
          if issubclass(cls, (
              parsers_interface.BaseParser, parsers_plugins.BasePlugin)):

            # TODO: detect corresponding dtFabric .yaml file
            parser_name = getattr(cls, 'NAME', None)
            if not parser_name or parser_name in self._PARSER_NAME_IGNORE_LIST:
              continue

            category = self._DATA_FORMAT_CATEGORY_PER_PACKAGE_PATH.get(
                package_path, 'File formats')

            data_format = getattr(cls, 'DATA_FORMAT', None)
            if not data_format:
              print(('WARNING: parser or plugin: {0:s} missing '
                     'DATA_FORMAT').format(parser_name))

            url = ''

            dtfabric_file = os.path.join(package_path, ''.join([name, '.yaml']))
            if os.path.exists(dtfabric_file):
              definitions_registry = (
                  dtfabric_registry.DataTypeDefinitionsRegistry())
              definitions_reader = (
                  dtfabric_reader.YAMLDataTypeDefinitionsFileReader())

              try:
                definitions_reader.ReadFile(definitions_registry, dtfabric_file)
                # TODO: determine the URL using definitions_registry.
              except Exception:  # pylint: disable=broad-except
                pass

            data_format_descriptor = DataFormatDescriptor(
                category=category, name=data_format, url=url)
            data_format_descriptors.append(data_format_descriptor)

    return data_format_descriptors

  def GetDataFormatInformation(self):
    """Retrieves data format information from Plaso.

    Returns:
      list[DataFormatDescriptor]: data format descriptors.
    """
    return self._GetDataFormatInformationFromPackage(['plaso'])


def Main():
  """The main program function.

  Returns:
    bool: True if successful or False if not.
  """
  argument_parser = argparse.ArgumentParser(description=(
      'Extract data format information from Plaso.'))

  # TODO: option to export for information on forensicswiki.
  # TODO: add information about supported compressed stream formats.
  # TODO: add information about supported archive file formats.

  argument_parser.parse_args()

  extractor = DataFormatInformationExtractor()

  data_format_descriptors = extractor.GetDataFormatInformation()
  if not data_format_descriptors:
    print('Unable to determine data format information')
    return False

  output_text = extractor.FormatDataFormats(data_format_descriptors)
  print(output_text)

  return True


if __name__ == '__main__':
  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the shared functionality for text file based output modules."""

import io
import unittest

from plaso.output import text_file

from tests.output import test_lib


class TextFileOutputModuleTest(test_lib.OutputModuleTestCase):
  """Tests for the shared functionality for text file based output modules."""

  # pylint: disable=protected-access

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    test_file_object = io.StringIO()

    output_mediator = self._CreateOutputMediator()
    output_module = text_file.TextFileOutputModule()
    output_module._file_object = test_file_object

    output_module.WriteHeader(output_mediator)

    header = test_file_object.getvalue()
    self.assertEqual(header, '')

  def testWriteFooter(self):
    """Tests the WriteFooter function."""
    test_file_object = io.StringIO()

    output_module = text_file.TextFileOutputModule()
    output_module._file_object = test_file_object

    output_module.WriteFooter()

    footer = test_file_object.getvalue()
    self.assertEqual(footer, '')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Symantec AV Log parser."""

import unittest

from plaso.parsers import symantec

from tests.parsers import test_lib


class SymantecAccessProtectionUnitTest(test_lib.ParserTestCase):
  """Tests for the Symantec AV Log parser."""

  # pylint: disable=protected-access

  def testParse(self):
    """Tests the Parse function."""
    parser = symantec.SymantecParser()
    storage_writer = self._ParseFile(['Symantec.Log'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 8)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'action0': '14',
        'action1': '5',
        'action2': '3',
        'cat': '1',
        'data_type': 'av:symantec:scanlog',
        'event': '5',
        'event_data': '201\t4\t6\t1\t65542\t0\t0\t0\t0\t0\t0',
        'file': 'D:\\Twinkle_Prod$\\VM11 XXX\\outside\\test.exe.txt',
        'last_written_time': '2012-11-30T10:47:29',
        'scanid': '0',
        'user': 'davnads',
        'virus': 'W32.Changeup!gen33'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Script to generate Windows time zone name to Python mappings."""

import argparse
import sys

import urllib.request as urllib_request

from defusedxml import ElementTree


def Main():
  """The main program function.

  Returns:
    bool: True if successful or False if not.
  """
  argument_parser = argparse.ArgumentParser(description=(
      'Generate Windows time zone name to Python mappings.'))

  argument_parser.parse_args()

  url = ('https://raw.githubusercontent.com/unicode-org/cldr/master/common/'
         'supplemental/windowsZones.xml')
  with urllib_request.urlopen(url) as request_object:
    windows_time_zones_xml_data = request_object.read()

  windows_time_zones_xml_data = windows_time_zones_xml_data.decode('utf-8')

  windows_time_zones_xml = ElementTree.fromstring(windows_time_zones_xml_data)

  time_zone_mappings = {
    'Kamchatka Standard Time': 'Asia/Kamchatka',
    'Mexico Standard Time 2': 'America/Chihuahua',
    'Mexico Standard Time': 'America/Mexico_City',
    'Mid-Atlantic Standard Time': 'America/New_York'}
  for windows_time_zone in windows_time_zones_xml.findall('.//mapZone'):
    if windows_time_zone.attrib['territory'] == '001':
      windows_name = windows_time_zone.attrib['other']
      time_zone_mappings[windows_name] = windows_time_zone.attrib['type']

  for windows_name, python_name in sorted(time_zone_mappings.items()):
    print('    \'{0:s}\': \'{1:s}\''.format(windows_name, python_name))

  return True


if __name__ == '__main__':
  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Run Windows Registry plugin."""

import unittest

from plaso.parsers.winreg_plugins import run

from tests.parsers.winreg_plugins import test_lib


class AutoRunsPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the auto rus Windows Registry plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = run.AutoRunsPlugin()

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Run')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunOnce')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Run')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunOnce')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunOnce\\Setup')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunServices')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunServicesOnce')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  def testProcessNtuserRun(self):
    """Tests the Process function on a Run key."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-RunTests.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Run')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = run.AutoRunsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:run',
        'entries': [
            'Sidebar: %ProgramFiles%\\Windows Sidebar\\Sidebar.exe /autoRun'],
        'key_path': key_path,
        'last_written_time': '2012-04-05T17:03:53.9920616+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessNtuserRunOnce(self):
    """Tests the Process function on a Run key."""
    test_file_entry = self._GetTestFileEntry(['NTUSER-RunTests.DAT'])
    key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunOnce')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = run.AutoRunsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:run',
        'entries': [
            'mctadmin: C:\\Windows\\System32\\mctadmin.exe'],
        'key_path': key_path,
        'last_written_time': '2012-04-05T17:03:53.9920616+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessSoftwareRun(self):
    """Tests the Process function on a Run key."""
    test_file_entry = self._GetTestFileEntry(['SOFTWARE-RunTests'])
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'Run')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = run.AutoRunsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:run',
        'entries': [
            ('McAfee Host Intrusion Prevention Tray: "C:\\Program Files\\'
             'McAfee\\Host Intrusion Prevention\\FireTray.exe"'),
            ('VMware Tools: "C:\\Program Files\\VMware\\VMware Tools\\'
             'VMwareTray.exe"'),
            ('VMware User Process: "C:\\Program Files\\VMware\\VMware Tools\\'
             'VMwareUser.exe"')],
        'key_path': key_path,
        'last_written_time': '2011-09-16T20:57:09.0675758+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessSoftwareRunOnce(self):
    """Tests the Process function on a RunOnce key."""
    test_file_entry = self._GetTestFileEntry(['SOFTWARE-RunTests'])
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        'RunOnce')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = run.AutoRunsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'windows:registry:run',
        'entries': [
            '*WerKernelReporting: %SYSTEMROOT%\\SYSTEM32\\WerFault.exe -k -rq'],
        'key_path': key_path,
        'last_written_time': '2012-04-06T14:07:27.7500000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows Registry custom event formatter helpers."""

import unittest

from plaso.formatters import winreg

from tests.formatters import test_lib


class WindowsRegistryValuesFormatterHelperTest(
    test_lib.EventFormatterTestCase):
  """Tests for the Windows Registry values formatter helper."""

  def testFormatEventValues(self):
    """Tests the FormatEventValues function."""
    formatter_helper = winreg.WindowsRegistryValuesFormatterHelper()

    output_mediator = self._CreateOutputMediator()

    event_values = {'values': 'value1, value2'}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['values'], 'value1, value2')

    event_values = {'values': None}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['values'], '(empty)')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Android SMS plugin."""

import unittest

from plaso.parsers.sqlite_plugins import android_sms

from tests.parsers.sqlite_plugins import test_lib


class AndroidSMSTest(test_lib.SQLitePluginTestCase):
  """Tests for the Android SMS database plugin."""

  def testProcess(self):
    """Test the Process function on an Android SMS mmssms.db file."""
    plugin = android_sms.AndroidSMSPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(['mmssms.db'], plugin)

    # The SMS database file contains 9 events (5 SENT, 4 RECEIVED messages).
    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'address': '1 555-521-5554',
        'body': 'Yo Fred this is my new number.',
        'creation_time': '2013-10-29T16:56:28.038+00:00',
        'data_type': 'android:messaging:sms',
        'sms_type': 2,
        'sms_read': 1}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Snort3/Suricata fast-log alert log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import snort_fastlog

from tests.parsers.text_plugins import test_lib


class SnortFastLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for Snort3/Suricata fast-log alert log text parser plugin."""

  def testProcessWithSnort3Log(self):
    """Tests the Process function with a Snort fast-log alert log file."""
    plugin = snort_fastlog.SnortFastLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['snort3_alert_fast.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'classification': None,
        'data_type': 'snort:fastlog:alert',
        'destination_ip': '2001:4860:4860:0:0:0:0:8888',
        'message': 'PROTOCOL-ICMP PING Unix',
        'last_written_time': '0000-12-28T12:55:38.765402',
        'priority': 3,
        'protocol': 'ICMP',
        'rule_identifier': '1:366:11',
        'source_ip': '2001:df1:c200:c:0:0:0:35'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithSuricataLog(self):
    """Tests the Process function with a Suricata fast-log alert log file."""
    plugin = snort_fastlog.SnortFastLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['suricata_alert_fast.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 5)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'classification': 'Web Application Attack',
        'data_type': 'snort:fastlog:alert',
        'destination_ip': '192.168.1.4',
        'destination_port': 56068,
        'last_written_time': '2010-05-10T10:08:59.667372',
        'message': (
            'ET WEB_CLIENT ACTIVEX iDefense COMRaider ActiveX Control '
            'Arbitrary File Deletion'),
        'priority': 3,
        'protocol': 'TCP',
        'rule_identifier': '1:2009187:4',
        'source_ip': '11.11.232.144',
        'source_port': 80}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome autofill entries database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import chrome_autofill

from tests.parsers.sqlite_plugins import test_lib


class ChromeAutofillPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Google Chrome autofill entries database plugin."""

  def testProcess(self):
    """Tests the Process function on a Chrome autofill entries database."""
    plugin = chrome_autofill.ChromeAutofillPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(['Web Data'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2018-08-17T19:35:51+00:00',
        'data_type': 'chrome:autofill:entry',
        'field_name': 'repo',
        'last_used_time': '2018-08-17T19:35:51+00:00',
        'usage_count': 1,
        'value': 'log2timeline/plaso'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file imports Python modules that register parsers."""

from plaso.parsers import android_app_usage
from plaso.parsers import asl
from plaso.parsers import bencode_parser
from plaso.parsers import bodyfile
from plaso.parsers import bsm
from plaso.parsers import chrome_cache
from plaso.parsers import chrome_preferences
from plaso.parsers import cups_ipp
from plaso.parsers import custom_destinations
from plaso.parsers import czip
from plaso.parsers import esedb
from plaso.parsers import filestat
from plaso.parsers import firefox_cache
from plaso.parsers import fish_history
from plaso.parsers import fseventsd
from plaso.parsers import java_idx
from plaso.parsers import jsonl_parser
from plaso.parsers import locate
from plaso.parsers import macos_keychain
from plaso.parsers import mcafeeav
from plaso.parsers import msiecf
from plaso.parsers import networkminer
from plaso.parsers import ntfs
from plaso.parsers import olecf
from plaso.parsers import onedrive
from plaso.parsers import opera
from plaso.parsers import pe
from plaso.parsers import plist
from plaso.parsers import pls_recall
from plaso.parsers import recycler
from plaso.parsers import safari_cookies
from plaso.parsers import spotlight_storedb
from plaso.parsers import sqlite
from plaso.parsers import symantec
from plaso.parsers import systemd_journal
from plaso.parsers import text_parser
from plaso.parsers import trendmicroav
from plaso.parsers import unified_logging
from plaso.parsers import utmp
from plaso.parsers import utmpx
from plaso.parsers import wincc
from plaso.parsers import windefender_history
from plaso.parsers import winevt
from plaso.parsers import winevtx
from plaso.parsers import winjob
from plaso.parsers import winlnk
from plaso.parsers import winpca
from plaso.parsers import winprefetch
from plaso.parsers import winreg_parser
from plaso.parsers import winrestore

# Register parser plugins.
from plaso.parsers import bencode_plugins
from plaso.parsers import czip_plugins
from plaso.parsers import esedb_plugins
from plaso.parsers import jsonl_plugins
from plaso.parsers import olecf_plugins
from plaso.parsers import plist_plugins
from plaso.parsers import sqlite_plugins
from plaso.parsers import text_plugins
from plaso.parsers import winreg_plugins

# These modules do not register parsers themselves, but contain super classes
# used by parsers in other modules.
# from plaso.parsers import dsv_parser



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Software Update plist plugin."""

import unittest

from plaso.parsers.plist_plugins import software_update

from tests.parsers.plist_plugins import test_lib


class MacOSSoftwareUpdatePlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the SoftwareUpdate plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'com.apple.SoftwareUpdate.plist'

    plugin = software_update.MacOSSoftwareUpdatePlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:software_updata:entry',
        'full_update_time': '2014-01-06T17:43:48.000000+00:00',
        'recommended_updates': ['RAWCameraUpdate5.03 (031-2664)'],
        'system_version': '10.9.1 (13B42)',
        'update_time': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the iOS identityservices plist plugin."""

import unittest

from plaso.parsers.plist_plugins import ios_identityservices

from tests.parsers.plist_plugins import test_lib


class IOSIdstatusachePlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the iOS identityservices plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'com.apple.identityservices.idstatuscache.plist'

    plugin = ios_identityservices.IOSIdstatusachePlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 40)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'apple_identifier': 'tel:+13233208923',
        'data_type': 'ios:idstatuscache:lookup',
        'lookup_time': '2021-02-20T01:00:30.799723+00:00',
        'process_name': 'com.apple.private.alloy.tincan.audio'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 9)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Analysis plugin related functions and classes for testing."""

from plaso.analysis import mediator as analysis_mediator
from plaso.containers import events
from plaso.engine import timeliner
from plaso.parsers import interface as parsers_interface
from plaso.parsers import mediator as parsers_mediator
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib


class AnalysisPluginTestCase(shared_test_lib.BaseTestCase):
  """The unit test case for an analysis plugin."""

  def _AnalyzeEvents(
      self, event_values_list, plugin, user_accounts=None):
    """Analyzes events using the analysis plugin.

    Args:
      event_values_list (list[dict[str, object]]): list of event values.
      plugin (AnalysisPlugin): plugin.
      user_accounts (Optional[list[UserAccountArtifact]]): user accounts.

    Returns:
      FakeStorageWriter: storage writer.
    """
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    for user_account in user_accounts or []:
      storage_writer.AddAttributeContainer(user_account)

    test_events = []
    for event, event_data, event_data_stream in (
        containers_test_lib.CreateEventsFromValues(event_values_list)):
      storage_writer.AddAttributeContainer(event_data_stream)

      event_data.SetEventDataStreamIdentifier(event_data_stream.GetIdentifier())
      storage_writer.AddAttributeContainer(event_data)

      event.SetEventDataIdentifier(event_data.GetIdentifier())
      storage_writer.AddAttributeContainer(event)

      test_events.append((event, event_data, event_data_stream))

    mediator = analysis_mediator.AnalysisMediator(
        user_accounts=user_accounts)
    mediator.SetStorageWriter(storage_writer)

    for event, event_data, event_data_stream in test_events:
      plugin.ExamineEvent(mediator, event, event_data, event_data_stream)

    analysis_report = plugin.CompileReport(mediator)
    storage_writer.AddAttributeContainer(analysis_report)

    return storage_writer

  def _ParseAndAnalyzeFile(self, path_segments, parser, plugin):
    """Parses and analyzes a file using the parser and analysis plugin.

    Args:
      path_segments (list[str]): path segments inside the test data directory.
      parser (BaseParser): parser.
      plugin (AnalysisPlugin): plugin.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      SkipTest: if the path inside the test data directory does not exist and
          the test should be skipped.
    """
    storage_writer = self._ParseFile(path_segments, parser)
    mediator = analysis_mediator.AnalysisMediator()
    mediator.SetStorageWriter(storage_writer)

    for event in storage_writer.GetSortedEvents():
      event_data = None
      event_data_identifier = event.GetEventDataIdentifier()
      if event_data_identifier:
        event_data = storage_writer.GetAttributeContainerByIdentifier(
            events.EventData.CONTAINER_TYPE, event_data_identifier)

      event_data_stream = None
      if event_data:
        event_data_stream_identifier = event_data.GetEventDataStreamIdentifier()
        if event_data_stream_identifier:
          event_data_stream = storage_writer.GetAttributeContainerByIdentifier(
              events.EventDataStream.CONTAINER_TYPE,
              event_data_stream_identifier)

      plugin.ExamineEvent(mediator, event, event_data, event_data_stream)

    analysis_report = plugin.CompileReport(mediator)
    storage_writer.AddAttributeContainer(analysis_report)

    return storage_writer

  def _ParseFile(self, path_segments, parser):
    """Parses a file using the parser.

    Args:
      path_segments (list[str]): path segments inside the test data directory.
      parser (BaseParser): parser.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      SkipTest: if the path inside the test data directory does not exist and
          the test should be skipped.
    """
    parser_mediator = parsers_mediator.ParserMediator()

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()
    parser_mediator.SetStorageWriter(storage_writer)

    file_entry = self._GetTestFileEntry(path_segments)
    parser_mediator.SetFileEntry(file_entry)

    event_data_stream = events.EventDataStream()
    parser_mediator.ProduceEventDataStream(event_data_stream)

    if isinstance(parser, parsers_interface.FileEntryParser):
      parser.Parse(parser_mediator)

    elif isinstance(parser, parsers_interface.FileObjectParser):
      file_object = file_entry.GetFileObject()
      parser.Parse(parser_mediator, file_object)

    else:
      self.fail('Got unexpected parser type: {0!s}'.format(type(parser)))

    self._ProcessEventData(storage_writer)

    return storage_writer

  def _ProcessEventData(self, storage_writer):
    """Generate events from event data.

    Args:
      storage_writer (StorageWriter): storage writer.
    """
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.DATA_PATH)

    event_data = storage_writer.GetFirstWrittenEventData()
    while event_data:
      event_data_stream_identifier = event_data.GetEventDataStreamIdentifier()

      event_data_stream = None
      if event_data_stream_identifier:
        event_data_stream = storage_writer.GetAttributeContainerByIdentifier(
            events.EventDataStream.CONTAINER_TYPE, event_data_stream_identifier)

      event_data_timeliner.ProcessEventData(
          storage_writer, event_data, event_data_stream)

      event_data = storage_writer.GetNextWrittenEventData()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS TCC plugin."""

import unittest

from plaso.parsers.sqlite_plugins import macos_tcc

from tests.parsers.sqlite_plugins import test_lib


class MacOSTCCPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the MacOS TCC plugin."""

  def testProcess(self):
    """Tests the Process function on a MacOS TCC file."""
    plugin = macos_tcc.MacOSTCCPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(['TCC-test.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 21)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'allowed': 1,
        'client': 'com.apple.weather',
        'data_type': 'macos:tcc_entry',
        'modification_time': '2020-05-29T12:09:51+00:00',
        'prompt_count': 1,
        'service': 'kTCCServiceUbiquity'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the SQLite parser plugin for iOS netusage database files."""

import unittest

from plaso.parsers.sqlite_plugins import ios_netusage

from tests.parsers.sqlite_plugins import test_lib


class IOSNetusagePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the SQLite parser plugin for iOS netusage database files."""

  def testProcess(self):
    """Test the Process function."""
    plugin = ios_netusage.IOSNetusagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['netusage.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 384)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test a route database entry.
    expected_event_values = {
        'bytes_in': 131926734,
        'bytes_out': 59786697,
        'data_type': 'ios:netusage:route',
        'network_identifier': 'Carrier-18005506',
        'network_signature': '353537613832343930343563356433642D33613937333034',
        'network_type': 2,
        'start_time': '2021-02-19T05:00:00.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test a process database entry.
    expected_event_values = {
        'data_type': 'ios:netusage:process',
        'process_name': 'mDNSResponder',
        'start_time': '2021-01-17T19:23:38.212656+00:00',
        'wifi_in': 79188353,
        'wifi_out': 35398277,
        'wired_in': 12198756,
        'wired_out': 10841,
        'wireless_wan_in': 300997,
        'wireless_wan_out': 938811}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 50)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the SQLite parser plugin for iOS powerlog database files."""

import unittest

from plaso.parsers.sqlite_plugins import ios_powerlog

from tests.parsers.sqlite_plugins import test_lib


class IOSPowerlogApplicationUsagePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the SQLite parser plugin for iOS powerlog database files."""

  def testParse(self):
    """Tests the ParseApplicationRunTime method."""
    plugin = ios_powerlog.IOSPowerlogApplicationUsagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['powerlog_2021-12-16_05-54_84E2141B.PLSQL'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 102)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'background_time':  0.0,
        'bundle_identifier': 'com.apple.springboard.home-screen',
        'screen_on_time': 30.339694,
        'start_time': '2021-11-02T10:05:45+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the year-less log format helper mix-in."""

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.lib import yearless_helper
from plaso.parsers import mediator as parsers_mediator

from tests import test_lib as shared_test_lib


class YearLessLogFormatHelperTest(shared_test_lib.BaseTestCase):
  """Year-less log format definition helper mix-in tests."""

  # pylint: disable=protected-access

  def testGetYearsFromFileEntry(self):
    """Tests the _GetYearsFromFileEntry function."""
    test_path = self._GetTestFilePath(['syslog.gz'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)
    file_entry = path_spec_resolver.Resolver.OpenFileEntry(gzip_path_spec)

    test_helper = yearless_helper.YearLessLogFormatHelper()

    years = test_helper._GetYearsFromFileEntry(file_entry)
    self.assertEqual(years, set([2012]))

  def testGetMonthFromString(self):
    """Tests the _GetMonthFromString function."""
    test_helper = yearless_helper.YearLessLogFormatHelper()

    month = test_helper._GetMonthFromString('jan')
    self.assertEqual(month, 1)

    month = test_helper._GetMonthFromString('bogus')
    self.assertIsNone(month)

  def testGetRelativeYear(self):
    """Tests the _GetRelativeYear function."""
    test_helper = yearless_helper.YearLessLogFormatHelper()

    test_helper._SetMonthAndYear(11, 2022)

    relative_year = test_helper._GetRelativeYear()
    self.assertEqual(relative_year, 0)

  def testGetYear(self):
    """Tests the _GetYear function."""
    test_helper = yearless_helper.YearLessLogFormatHelper()

    test_helper._SetMonthAndYear(11, 2022)

    year = test_helper._GetYear()
    self.assertEqual(year, 2022)

  def testSetEstimatedYear(self):
    """Tests the _SetEstimatedYear function."""
    parser_mediator = parsers_mediator.ParserMediator()

    test_path = self._GetTestFilePath(['syslog.gz'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)
    file_entry = path_spec_resolver.Resolver.OpenFileEntry(gzip_path_spec)

    parser_mediator.SetFileEntry(file_entry)

    test_helper = yearless_helper.YearLessLogFormatHelper()

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertIsNone(test_helper._month)

    test_helper._SetEstimatedYear(parser_mediator)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 2012)
    self.assertIsNone(test_helper._month)

    test_path = self._GetTestFilePath(['syslog.xz'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    file_entry = path_spec_resolver.Resolver.OpenFileEntry(os_path_spec)
    years = test_helper._GetYearsFromFileEntry(file_entry)

    compressed_stream_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_COMPRESSED_STREAM,
        compression_method=dfvfs_definitions.COMPRESSION_METHOD_XZ,
        parent=os_path_spec)
    file_entry = path_spec_resolver.Resolver.OpenFileEntry(
        compressed_stream_path_spec)

    parser_mediator.SetFileEntry(file_entry)

    test_helper = yearless_helper.YearLessLogFormatHelper()

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertIsNone(test_helper._month)

    test_helper._SetEstimatedYear(parser_mediator)

    expected_year = min(years)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, expected_year)
    self.assertIsNone(test_helper._month)

  def testSetMonthAndYear(self):
    """Tests the _SetMonthAndYear function."""
    test_helper = yearless_helper.YearLessLogFormatHelper()

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertIsNone(test_helper._month)

    test_helper._SetMonthAndYear(11, 2022)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 2022)
    self.assertEqual(test_helper._month, 11)

  def testUpdateYear(self):
    """Tests the _UpdateYear function."""
    test_helper = yearless_helper.YearLessLogFormatHelper()

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertIsNone(test_helper._month)

    test_helper._UpdateYear(1)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertEqual(test_helper._month, 1)

    test_helper._UpdateYear(5)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertEqual(test_helper._month, 5)

    test_helper._UpdateYear(12)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertEqual(test_helper._month, 12)

    test_helper._UpdateYear(1)

    self.assertEqual(test_helper._relative_year, 1)
    self.assertEqual(test_helper._year, 1)
    self.assertEqual(test_helper._month, 1)

    test_helper._UpdateYear(12)

    self.assertEqual(test_helper._relative_year, 0)
    self.assertEqual(test_helper._year, 0)
    self.assertEqual(test_helper._month, 12)

    test_helper._UpdateYear(5)

    self.assertEqual(test_helper._relative_year, 1)
    self.assertEqual(test_helper._year, 1)
    self.assertEqual(test_helper._month, 5)

    test_helper._UpdateYear(1)

    self.assertEqual(test_helper._relative_year, 2)
    self.assertEqual(test_helper._year, 2)
    self.assertEqual(test_helper._month, 1)

  def testGetYearLessLogHelper(self):
    """Tests the GetYearLessLogHelper function."""
    test_helper = yearless_helper.YearLessLogFormatHelper()

    year_less_log_helper = test_helper.GetYearLessLogHelper()
    self.assertIsNotNone(year_less_log_helper)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Winlogon Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import winlogon

from tests.parsers.winreg_plugins import test_lib


class WinlogonPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Winlogon Windows Registry plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'Winlogon', key_path=key_path,
        last_written_time=filetime.timestamp, offset=153)

    # Setup Winlogon values.
    value_data = '1'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'AutoAdminLogon', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'AutoRestartShell', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = '0 0 0'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Background', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = '10'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'CachedLogonsCount', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'no'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DebugServerCommand', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = ''.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DefaultDomainName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'user'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DefaultUserName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x01'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DisableCAD', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ForceUnlockLogon', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = ''.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'LegalNoticeCaption', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = ''.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'LegalNoticeText', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x05'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'PasswordExpiryWarning', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = '0'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'PowerdownAfterShutdown', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = '{A520A1A4-1780-4FF6-BD18-167343C5AF16}'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'PreCreateKnownFolders', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = '1'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ReportBootOk', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'explorer.exe'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Shell', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x2b'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ShutdownFlags', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    registry_key.AddValue(registry_value)

    value_data = '0'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ShutdownWithoutLogon', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'C:\\Windows\\system32\\userinit.exe'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Userinit', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = 'SystemPropertiesPerformance.exe/pagefile'.encode(
        'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'VMApplet', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    value_data = '0'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'WinStationsDisabled', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    registry_key.AddValue(registry_value)

    # Setup registered event handlers.
    notify_key_name = 'Notify'
    notify_key = dfwinreg_fake.FakeWinRegistryKey(notify_key_name)
    registry_key.AddSubkey(notify_key_name, notify_key)

    navlogon_key_name = 'NavLogon'
    navlogon_key = dfwinreg_fake.FakeWinRegistryKey(
        navlogon_key_name, last_written_time=filetime.timestamp)
    notify_key.AddSubkey(navlogon_key_name, navlogon_key)

    value_data = 'NavLogon.dll'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DllName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    navlogon_key.AddValue(registry_value)

    value_data = 'NavLogoffEvent'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Logoff', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    navlogon_key.AddValue(registry_value)

    value_data = 'NavStartShellEvent'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'StartShell', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    navlogon_key.AddValue(registry_value)

    secret_malware_key_name = 'SecretMalware'
    secret_malware_key = dfwinreg_fake.FakeWinRegistryKey(
        secret_malware_key_name, last_written_time=filetime.timestamp)
    notify_key.AddSubkey(secret_malware_key_name, secret_malware_key)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Asynchronous', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secret_malware.dll'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DllName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Impersonate', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventLock'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Lock', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventLogoff'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Logoff', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventLogon'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Logon', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventShutdown'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Shutdown', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventSmartCardLogonNotify'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'SmartCardLogonNotify', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventStartShell'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'StartShell', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventStartup'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Startup', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventStopScreenSaver'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'StopScreenSaver', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    value_data = 'secretEventUnlock'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Unlock', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    secret_malware_key.AddValue(registry_value)

    return registry_key

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = winlogon.WinlogonPlugin()

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion\\'
        'Winlogon')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  def testProcess(self):
    """Tests the Process function on created key."""
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion')
    time_string = '2013-01-30 10:47:57'
    registry_key = self._CreateTestKey(key_path, time_string)

    plugin = winlogon.WinlogonPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 14)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application': 'VmApplet',
        'command': 'SystemPropertiesPerformance.exe/pagefile',
        'data_type': 'windows:registry:winlogon',
        'key_path': key_path,
        'last_written_time': '2013-01-30T10:47:57.0000000+00:00',
        'trigger': 'Logon'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'application': 'NavLogon',
        'command': 'NavLogon.dll',
        'data_type': 'windows:registry:winlogon',
        'key_path': '{0:s}\\Notify\\NavLogon'.format(key_path),
        'last_written_time': '2013-01-30T10:47:57.0000000+00:00',
        'trigger': 'Logoff'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for the Microsoft User Access Logging (UAL) ESE database.

User Access Logging (UAL) is present in Windows Server editions starting with
Window Server 2012.

Also see:
  https://www.crowdstrike.com/blog/user-access-logging-ual-overview/
"""

import ipaddress
import uuid

from dfvfs.resolver import resolver as path_spec_resolver
from dfvfs.path import factory as path_spec_factory

from plaso.containers import events
from plaso.parsers import esedb
from plaso.parsers.esedb_plugins import interface


class UserAccessLoggingClientsEventsData(events.EventData):
  """Windows User Access Logging CLIENTS table event data.

  Attributes:
    access_time (dfdatetime.DateTimeValues): last access date and time.
    authenticated_username (str): domain/user account name
        performing the access.
    client_name (str): client name, use unknown.
    insert_time (dfdatetime.DateTimeValues): date and time the entry was
        first inserted into the table.
    role_identifier (str): identifier of the service accessed.
    role_name (str): Name of the service accessed.
    source_ip_address (str): source IP address.
    tenant_identifier (str): unique identifier of a tenant client.
    total_accesses (int): Count of accesses for the year.
  """

  DATA_TYPE = 'windows:user_access_logging:clients'

  def __init__(self):
    """Initializes event data."""
    super(UserAccessLoggingClientsEventsData, self).__init__(
        data_type=self.DATA_TYPE)
    self.access_time = None
    self.authenticated_username = None
    self.client_name = None
    self.insert_time = None
    self.role_identifier = None
    self.role_name = None
    self.source_ip_address = None
    self.tenant_identifier = None
    self.total_accesses = None


class UserAccessLoggingDNSEventData(events.EventData):
  """Windows User Access Logging DNS table event data.

  Attributes:
    hostname (str): hostname.
    ip_address (str): IP address.
    last_seen_time (dfdatetime.DateTimeValues): date and time the hostname to
        IP address mapping was last observed.
  """

  DATA_TYPE = 'windows:user_access_logging:dns'

  def __init__(self):
    """Initializes event data."""
    super(UserAccessLoggingDNSEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.hostname = None
    self.ip_address = None
    self.last_seen_time = None


class UserAccessLoggingRoleAccessEventsData(events.EventData):
  """Windows User Access Logging ROLE_ACCESS table event data.

  Attributes:
    first_seen_time (dfdatetime.DateTimeValues): date and time the role was
        first observed to be used.
    last_seen_time (dfdatetime.DateTimeValues): date and time the role was
        last observed to be used.
    role_identifier (str): identifier of the role.
    role_name (str): name of the role.
  """

  DATA_TYPE = 'windows:user_access_logging:role_access'

  def __init__(self):
    """Initializes event data."""
    super(UserAccessLoggingRoleAccessEventsData, self).__init__(
        data_type=self.DATA_TYPE)
    self.first_seen_time = None
    self.last_seen_time = None
    self.role_identifier = None
    self.role_name = None


class UserAccessLoggingSystemIdentityEventdata(events.EventData):
  """Windows User Access Logging SYSTEM_IDENTITY table event data.

  Attributes:
    creation_time (dfdatetime.DateTimeValues): date and time the system
        identity was created.
    operating_system_build (int): operating system build.
    system_dns_hostname (str): System hostname.
    system_domain_name (str): System domain name.
  """

  DATA_TYPE = 'windows:user_access_logging:system_identity'

  def __init__(self):
    """Initializes event data."""
    super(UserAccessLoggingSystemIdentityEventdata, self).__init__(
        data_type=self.DATA_TYPE)
    self.creation_time = None
    self.operating_system_build = None
    self.system_dns_hostname = None
    self.system_domain_name = None


class UserAccessLoggingVirtualMachinesEventData(events.EventData):
  """Windows User Access Logging VIRTUALMACHINES table event data.

  Attributes:
    bios_identifier (str): BIOS identifier.
    creation_time (dfdatetime.DateTimeValues): date and time the virtual
        machine was created.
    last_active_time (dfdatetime.DateTimeValues): date and time the virtual
        machine was last observed to be active.
    serial_number (str): Serial number.
    vm_identifier (str): identifier of the virtual machine.
  """

  DATA_TYPE = 'windows:user_access_logging:virtual_machines'

  def __init__(self):
    """Initializes event data."""
    super(UserAccessLoggingVirtualMachinesEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.bios_identifier = None
    self.creation_time = None
    self.last_active_time = None
    self.serial_number = None
    self.vm_identifier = None


class UserAccessLoggingESEDBPlugin(interface.ESEDBPlugin):
  """Parses Windows User Access Logging ESE database file. """

  NAME = 'user_access_logging'
  DATA_FORMAT = 'Windows User Access Logging ESE database file'

  REQUIRED_TABLES = {
      'CLIENTS': 'ParseClientsTable',
      'DNS': 'ParseDNSTable',
      'ROLE_ACCESS': 'ParseRoleAccessTable',
      'VIRTUALMACHINES': 'ParseVirtualMachinesTable'}

  _CLIENTS_TABLE_VALUE_MAPPINGS = {
      'Address': '_ConvertIPAddressValue',
      'RoleGuid': '_ConvertGUIDToString',
      'TenantId': '_ConvertGUIDToString'}

  _DNS_TABLE_VALUE_MAPPINGS = {
      'Address': '_ConvertDNSAddressValue'}

  _VIRTUALMACHINES_TABLE_VALUE_MAPPINGS = {
      'BIOSGuid': '_ConvertGUIDToString',
      'VmGuid': '_ConvertGUIDToString'}

  _ROLE_ACCESS_TABLE_VALUE_MAPPINGS = {
      'RoleGuid': '_ConvertGUIDToString'}

  _ROLE_IDS_TABLE_VALUE_MAPPINGS = {
      'RoleGuid': '_ConvertGUIDToString'}

  def __init__(self):
    """Initializes an UAL ESE database file parser plugin."""
    super(UserAccessLoggingESEDBPlugin, self).__init__()
    self._role_mappings = {}

  def _ConvertDNSAddressValue(self, value):
    """Converts the address column value of a DNS table into a string.

    Args:
      value (bytes): DNS address.

    Returns:
      str: string representation of the DNS address.
    """
    return value.replace(b'\x00', b'').decode('utf-8')

  def _ConvertGUIDToString(self, value):
    """Converts a GUID into a string representation.

    Args:
      value (bytes): a little-endian GUID value.

    Returns:
      str: string representation of the GUID.
    """
    guid_value = uuid.UUID(bytes_le=value)
    return '{{{0!s}}}'.format(guid_value).lower()

  def _ConvertIPAddressValue(self, value):
    """Converts bytes representation of an IP to a string.

    Args:
     value (bytes): IP address.

    Returns:
      str: string representation of the IP address.
    """
    return str(ipaddress.ip_address(value))

  def ParseClientsTable(
      self, parser_mediator, database=None, table=None, **unused_kwargs):
    """Parses a CLIENTS table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      database (Optional[ESEDatabase]): ESE database.
      table (Optional[pyesedb.table]): table.

    Raises:
      ValueError: if the database or table value is missing.
    """
    if database is None:
      raise ValueError('Missing database value.')

    if table is None:
      raise ValueError('Missing table value.')

    if not self._role_mappings:
      system_identity_file_entry = self._GetSystemIdentityDatabase(
          parser_mediator)
      if system_identity_file_entry:
        self._ProcessSystemInformationDatabase(
            parser_mediator, system_identity_file_entry)

    for record_index, esedb_record in enumerate(table.records):
      if parser_mediator.abort:
        break

      try:
        record_values = self._GetRecordValues(
            parser_mediator, table.name, record_index, esedb_record,
            value_mappings=self._CLIENTS_TABLE_VALUE_MAPPINGS)
      except (UnicodeDecodeError, ValueError):
        parser_mediator.ProduceExtractionWarning((
            'Unable to retrieve record values from record: {0:d} '
            'in table: {1:s}').format(record_index, table.name))
        continue

      event_data = UserAccessLoggingClientsEventsData()
      event_data.access_time = self._GetFiletimeRecordValue(
          record_values, 'LastAccess')
      event_data.authenticated_username = record_values.get(
          'AuthenticatedUserName', None)
      event_data.client_name = record_values.get('ClientName', None)
      event_data.role_identifier = record_values.get('RoleGuid', None)
      event_data.insert_time = self._GetFiletimeRecordValue(
          record_values, 'InsertDate')
      event_data.role_name = self._role_mappings.get(
          event_data.role_identifier, 'Unknown')
      event_data.source_ip_address = record_values.get('Address', None)
      event_data.tenant_identifier = record_values.get('TenantId', None)
      event_data.total_accesses = record_values.get('TotalAccesses', None)

      parser_mediator.ProduceEventData(event_data)

  def ParseRoleAccessTable(
      self, parser_mediator, database=None, table=None, **unused_kwargs):
    """Parses a ROLE_ACCESS table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      database (Optional[ESEDatabase]): ESE database.
      table (Optional[pyesedb.table]): table.

    Raises:
      ValueError: if the database or table value is missing.
    """
    if database is None:
      raise ValueError('Missing database value.')

    if table is None:
      raise ValueError('Missing table value.')

    if not self._role_mappings:
      system_identity_file_object = self._GetSystemIdentityDatabase(
          parser_mediator)
      if system_identity_file_object:
        self._ProcessSystemInformationDatabase(
            parser_mediator, system_identity_file_object)

    for record_index, esedb_record in enumerate(table.records):
      if parser_mediator.abort:
        break

      try:
        record_values = self._GetRecordValues(
            parser_mediator, table.name, record_index, esedb_record,
            value_mappings=self._ROLE_ACCESS_TABLE_VALUE_MAPPINGS)
      except (UnicodeDecodeError, ValueError):
        parser_mediator.ProduceExtractionWarning((
            'Unable to retrieve record values from record: {0:d} '
            'in table: {1:s}').format(record_index, table.name))
        continue

      event_data = UserAccessLoggingRoleAccessEventsData()
      event_data.first_seen_time = self._GetFiletimeRecordValue(
          record_values, 'FirstSeen')
      event_data.last_seen_time = self._GetFiletimeRecordValue(
          record_values, 'LastSeen')
      event_data.role_identifier = record_values.get('RoleGuid', None)
      event_data.role_name = self._role_mappings.get(
          event_data.role_identifier, 'Unknown')

      parser_mediator.ProduceEventData(event_data)

  def ParseDNSTable(
      self, parser_mediator, database=None, table=None, **unused_kwargs):
    """Parses a DNS table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      database (Optional[ESEDatabase]): ESE database.
      table (Optional[pyesedb.table]): table.

    Raises:
      ValueError: if the database or table value is missing.
    """
    if database is None:
      raise ValueError('Missing database value.')

    if table is None:
      raise ValueError('Missing table value.')

    for record_index, esedb_record in enumerate(table.records):
      if parser_mediator.abort:
        break

      try:
        record_values = self._GetRecordValues(
            parser_mediator, table.name, record_index, esedb_record,
            value_mappings=self._DNS_TABLE_VALUE_MAPPINGS)
      except (UnicodeDecodeError, ValueError):
        parser_mediator.ProduceExtractionWarning((
            'Unable to retrieve record values from record: {0:d} '
            'in table: {1:s}').format(record_index, table.name))
        continue

      event_data = UserAccessLoggingDNSEventData()
      event_data.hostname = record_values.get('HostName', None)
      event_data.ip_address = record_values.get('Address', None)
      event_data.last_seen_time = self._GetFiletimeRecordValue(
          record_values, 'LastSeen')

      parser_mediator.ProduceEventData(event_data)

  def ParseVirtualMachinesTable(
      self, parser_mediator, database=None, table=None, **unused_kwargs):
    """Parses a VIRTUALMACHINES table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      database (Optional[ESEDatabase]): ESE database.
      table (Optional[pyesedb.table]): table.

    Raises:
      ValueError: if the database or table value is missing.
    """
    if database is None:
      raise ValueError('Missing database value.')

    if table is None:
      raise ValueError('Missing table value.')

    for record_index, esedb_record in enumerate(table.records):
      if parser_mediator.abort:
        break

      try:
        record_values = self._GetRecordValues(
            parser_mediator, table.name, record_index, esedb_record,
            value_mappings=self._VIRTUALMACHINES_TABLE_VALUE_MAPPINGS)
      except (UnicodeDecodeError, ValueError):
        parser_mediator.ProduceExtractionWarning((
            'Unable to retrieve record values from record: {0:d} '
            'in table: {1:s}').format(record_index, table.name))
        continue

      event_data = UserAccessLoggingVirtualMachinesEventData()
      event_data.bios_identifier = record_values.get('BIOSGuid', None)
      event_data.creation_time = self._GetFiletimeRecordValue(
          record_values, 'CreationTime')
      event_data.last_active_time = self._GetFiletimeRecordValue(
          record_values, 'LastSeenActive')
      event_data.serial_number = record_values.get('SerialNumber', None)
      event_data.vm_identifier = record_values.get('VMGuid', None)

      parser_mediator.ProduceEventData(event_data)

  def _GetSystemIdentityDatabase(self, parser_mediator):
    """Locate SystemIdentity.mdb.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.

    Returns:
      dfvfs.FileEntry: a file entry or None if the database cannot be located.
    """
    file_entry = parser_mediator.GetFileEntry()
    file_system = file_entry.GetFileSystem()
    path_segments = file_system.SplitPath(file_entry.path_spec.location)
    path_segments.pop()
    path_segments.append('SystemIdentity.mdb')

    kwargs = {}
    if file_entry.path_spec.parent:
      kwargs['parent'] = file_entry.path_spec.parent
    kwargs['location'] = file_system.JoinPath(path_segments)

    system_identity_file_path_spec = path_spec_factory.Factory.NewPathSpec(
        file_entry.path_spec.TYPE_INDICATOR, **kwargs)

    system_identity_file_entry = None
    try:
      system_identity_file_entry = path_spec_resolver.Resolver.OpenFileEntry(
          system_identity_file_path_spec)
    except RuntimeError as exception:
      message = (
          'Unable to open SystemIdentity.mdb file: {0:s} with error: '
          '{1!s}'.format(kwargs['location'], exception))
      parser_mediator.ProduceExtractionWarning(message)

    return system_identity_file_entry

  def _ProcessSystemInformationDatabase(self, parser_mediator, file_entry):
    """Process SystemIdentity.mdb and extract Role GUID -> Role name mappings.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_entry (dfvfs.FileEntry): a file entry
    """
    file_object = file_entry.GetFileObject()
    database = esedb.ESEDatabase()

    try:
      database.Open(file_object)
    except (IOError, ValueError) as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to open SystemInformation.mdb with error: {0!s}'.format(
              exception))
      return

    role_ids_table = database.GetTableByName('ROLE_IDS')
    if not role_ids_table:
      parser_mediator.ProduceExtractionWarning(
          'unable to get ROLE_IDS table in SystemInformation.mdb')
    else:
      self._ParseRoleIDsTable(
          parser_mediator, database=database, table=role_ids_table)

    system_identity_table = database.GetTableByName('SYSTEM_IDENTITY')
    if not system_identity_table:
      parser_mediator.ProduceExtractionWarning(
         'unable to get SYSTEM_IDENTITY table in SystemInformation.mdb')
    else:
      self._ParseSystemIdentityTable(
          parser_mediator, database=database, table=system_identity_table)

    database.Close()

  def _ParseRoleIDsTable(
      self, parser_mediator, database=None, table=None, **unused_kwargs):
    """Parses a SystemIdentity.mdb ROLE_IDS table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      database (Optional[ESEDatabase]): ESE database.
      table (Optional[pyesedb.table]): table.

    Raises:
      ValueError: if the database or table value is missing.
    """
    if database is None:
      raise ValueError('Missing database value.')

    if table is None:
      raise ValueError('Missing table value.')

    for record_index, esedb_record in enumerate(table.records):
      if parser_mediator.abort:
        break
      try:
        record_values = self._GetRecordValues(
            parser_mediator, table.name, record_index, esedb_record,
            value_mappings=self._ROLE_IDS_TABLE_VALUE_MAPPINGS)
      except (UnicodeDecodeError, ValueError):
        parser_mediator.ProduceExtractionWarning((
            'Unable to retrieve record values from record: {0:d} '
            'in table: {1:s}').format(record_index, table.name))
        continue

      role_identifier = record_values.get('RoleGuid', None)
      role_name = record_values.get('RoleName', None)
      if role_identifier and role_name:
        self._role_mappings[role_identifier] = role_name

  def _ParseSystemIdentityTable(
      self, parser_mediator, database=None, table=None, **unused_kwargs):
    """Parses a SystemIdentity.mdb SYSTEM_IDENTITY table.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      database (Optional[ESEDatabase]): ESE database.
      table (Optional[pyesedb.table]): table.

    Raises:
      ValueError: if the database or table value is missing.
    """
    if database is None:
      raise ValueError('Missing database value.')

    if table is None:
      raise ValueError('Missing table value.')

    for record_index, esedb_record in enumerate(table.records):
      if parser_mediator.abort:
        break
      try:
        record_values = self._GetRecordValues(
            parser_mediator, table.name, record_index, esedb_record)
      except (UnicodeDecodeError, ValueError):
        parser_mediator.ProduceExtractionWarning((
            'Unable to retrieve record values from record: {0:d} '
            'in table: {1:s}').format(record_index, table.name))
        continue

      event_data = UserAccessLoggingSystemIdentityEventdata()
      event_data.creation_time = self._GetFiletimeRecordValue(
          record_values, 'CreationTime')
      event_data.operating_system_build = record_values.get(
          'OSBuildNumber', None)
      event_data.system_dns_hostname = record_values.get(
          'SystemDNSHostName', None)
      event_data.system_domain_name = record_values.get(
          'SystemDomainName', None)

      parser_mediator.ProduceEventData(event_data)


esedb.ESEDBParser.RegisterPlugin(UserAccessLoggingESEDBPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS Mobile Installation log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import ios_sysdiag_log

from tests.parsers.text_plugins import test_lib


class IOSSysdiagLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the iOS Mobile Installation log text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = ios_sysdiag_log.IOSSysdiagLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['ios_sysdiag.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 28)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': (
            'Ignoring plugin at /System/Library/PrivateFrameworks/'
            'AccessibilityUtilities.framework/PlugIns/com.apple.accessibility.'
            'Accessibility.HearingAidsTapToRadar.appex due to validation '
            'issue(s). See previous log messages for details.'),
        'originating_call': (
            '+[MILaunchServicesDatabaseGatherer '
            'enumeratePluginKitPluginsInBundle:updatingPluginParentID:'
            'ensurePluginsAreExecutable:installProfiles:error:enumerator:]'),
        'process_identifier': 176,
        'severity': 'err',
        'written_time': '2021-08-11T05:51:02+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 7)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Chrome extension analysis plugin."""

import collections
import os
import unittest

from plaso.analysis import chrome_extension
from plaso.containers import artifacts
from plaso.containers import reports
from plaso.lib import definitions

from tests import test_lib as shared_test_lib
from tests.analysis import test_lib


class MockChromeExtensionPlugin(chrome_extension.ChromeExtensionPlugin):
  """Chrome extension analysis plugin used for testing."""

  NAME = 'chrome_extension_test'

  def _GetChromeWebStorePage(self, extension_identifier):
    """Retrieves the page for the extension from the Chrome store website.

    Args:
      extension_identifier (str): Chrome extension identifier.

    Returns:
      str: page content or None if not available.
    """
    chrome_web_store_file = shared_test_lib.GetTestFilePath([
        'chrome_extensions', extension_identifier])
    if not os.path.exists(chrome_web_store_file):
      return None

    with open(chrome_web_store_file, 'rb') as file_object:
      page_content = file_object.read()

    return page_content.decode('utf-8')


class ChromeExtensionTest(test_lib.AnalysisPluginTestCase):
  """Tests for the Chrome extension analysis plugin."""

  # pylint: disable=protected-access

  _MACOS_PATHS = [
      '/Users/dude/Library/Application Data/Google/Chrome/Default/Extensions',
      ('/Users/dude/Library/Application Data/Google/Chrome/Default/Extensions/'
       'apdfllckaahabafndbhieahigkjlhalf'),
      '/private/var/log/system.log',
      '/Users/frank/Library/Application Data/Google/Chrome/Default',
      '/Users/hans/Library/Application Data/Google/Chrome/Default',
      ('/Users/frank/Library/Application Data/Google/Chrome/Default/'
       'Extensions/pjkljhegncpnkpknbcohdijeoejaedia'),
      '/Users/frank/Library/Application Data/Google/Chrome/Default/Extensions']

  _MACOS_TEST_EVENTS = [
      {'_parser_chain': 'filestat',
       'data_type': 'fs:stat',
       'filename': path,
       'timestamp': '2015-01-01 17:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}
      for path in _MACOS_PATHS]

  _WINDOWS_PATHS = [
      'C:\\Users\\Dude\\SomeFolder\\Chrome\\Default\\Extensions',
      ('C:\\Users\\Dude\\SomeNoneStandardFolder\\Chrome\\Default\\Extensions\\'
       'hmjkmjkepdijhoojdojkdfohbdgmmhki'),
      ('C:\\Users\\frank\\AppData\\Local\\Google\\Chrome\\Extensions\\'
       'blpcfgokakmgnkcojhhkbfbldkacnbeo'),
      'C:\\Users\\frank\\AppData\\Local\\Google\\Chrome\\Extensions',
      ('C:\\Users\\frank\\AppData\\Local\\Google\\Chrome\\Extensions\\'
       'icppfcnhkcmnfdhfhphakoifcfokfdhg'),
      'C:\\Windows\\System32',
      'C:\\Stuff/with path separator\\Folder']

  _WINDOWS_TEST_EVENTS = [
      {'_parser_chain': 'filestat',
       'data_type': 'fs:stat',
       'filename': path,
       'timestamp': '2015-01-01 17:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}
      for path in _WINDOWS_PATHS]

  def testGetPathSegmentSeparator(self):
    """Tests the _GetPathSegmentSeparator function."""
    test_file_path = self._GetTestFilePath(['chrome_extensions'])
    self._SkipIfPathNotExists(test_file_path)

    plugin = MockChromeExtensionPlugin()

    for path in self._MACOS_PATHS:
      path_segment_separator = plugin._GetPathSegmentSeparator(path)
      self.assertEqual(path_segment_separator, '/')

    for path in self._WINDOWS_PATHS:
      path_segment_separator = plugin._GetPathSegmentSeparator(path)
      self.assertEqual(path_segment_separator, '\\')

  def testExamineEventAndCompileReportMacOSPaths(self):
    """Tests the ExamineEvent and CompileReport functions on MacOS paths."""
    test_file_path = self._GetTestFilePath(['chrome_extensions'])
    self._SkipIfPathNotExists(test_file_path)

    user_accounts = [
        artifacts.UserAccountArtifact(
            identifier='0', user_directory='/var/root', username='root'),
        artifacts.UserAccountArtifact(
            identifier='1123', user_directory='/Users/dude', username='dude'),
        artifacts.UserAccountArtifact(
            identifier='4052', user_directory='/Users/frank', username='frank'),
        artifacts.UserAccountArtifact(
            identifier='4352', user_directory='/Users/hans', username='hans')]

    plugin = MockChromeExtensionPlugin()
    storage_writer = self._AnalyzeEvents(
        self._MACOS_TEST_EVENTS, plugin, user_accounts=user_accounts)

    analysis_results = list(storage_writer.GetAttributeContainers(
        'chrome_extension_analysis_result'))
    self.assertEqual(len(analysis_results), 2)

    analysis_result = analysis_results[0]
    self.assertEqual(analysis_result.extension, 'Google Drive')
    self.assertEqual(
        analysis_result.extension_identifier,
        'apdfllckaahabafndbhieahigkjlhalf')
    self.assertEqual(analysis_result.username, 'dude')

    number_of_reports = storage_writer.GetNumberOfAttributeContainers(
        'analysis_report')
    self.assertEqual(number_of_reports, 1)

    analysis_report = storage_writer.GetAttributeContainerByIndex(
        reports.AnalysisReport.CONTAINER_TYPE, 0)
    self.assertIsNotNone(analysis_report)

    self.assertEqual(analysis_report.plugin_name, 'chrome_extension_test')

    expected_analysis_counter = collections.Counter({
        'dude': 1,
        'frank': 1})
    self.assertEqual(
        analysis_report.analysis_counter, expected_analysis_counter)

  def testExamineEventAndCompileReportWindowsPaths(self):
    """Tests the ExamineEvent and CompileReport functions on Windows paths."""
    test_file_path = self._GetTestFilePath(['chrome_extensions'])
    self._SkipIfPathNotExists(test_file_path)

    user_accounts = [
        artifacts.UserAccountArtifact(
            identifier='S-1', path_separator='\\',
            user_directory='C:\\Users\\dude', username='dude'),
        artifacts.UserAccountArtifact(
            identifier='S-2', path_separator='\\',
            user_directory='C:\\Users\\frank', username='frank')]

    plugin = MockChromeExtensionPlugin()
    storage_writer = self._AnalyzeEvents(
        self._WINDOWS_TEST_EVENTS, plugin, user_accounts=user_accounts)

    analysis_results = list(storage_writer.GetAttributeContainers(
        'chrome_extension_analysis_result'))
    self.assertEqual(len(analysis_results), 3)

    analysis_result = analysis_results[0]
    self.assertEqual(analysis_result.extension, 'Google Keep - notes and lists')
    self.assertEqual(
        analysis_result.extension_identifier,
        'hmjkmjkepdijhoojdojkdfohbdgmmhki')
    self.assertEqual(analysis_result.username, 'dude')

    number_of_reports = storage_writer.GetNumberOfAttributeContainers(
        'analysis_report')
    self.assertEqual(number_of_reports, 1)

    analysis_report = storage_writer.GetAttributeContainerByIndex(
        reports.AnalysisReport.CONTAINER_TYPE, 0)
    self.assertIsNotNone(analysis_report)

    self.assertEqual(analysis_report.plugin_name, 'chrome_extension_test')

    expected_analysis_counter = collections.Counter({
        'dude': 1,
        'frank': 2})
    self.assertEqual(
        analysis_report.analysis_counter, expected_analysis_counter)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Chrome Cache files parser."""

import unittest

from plaso.parsers import chrome_cache

from tests.parsers import test_lib


class ChromeCacheParserTest(test_lib.ParserTestCase):
  """Tests for the Chrome Cache files parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = chrome_cache.ChromeCacheParser()
    storage_writer = self._ParseFile(['chrome_cache', 'index'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 217)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2014-04-30T16:44:36.226091+00:00',
        'data_type': 'chrome:cache:entry',
        'original_url': (
            'https://s.ytimg.com/yts/imgbin/player-common-vfliLfqPT.webp')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testParseWithVersion3(self):
    """Tests the Parse function with version 3 Chrome Cache."""
    parser = chrome_cache.ChromeCacheParser()
    storage_writer = self._ParseFile(['chrome_cache_v3', 'index'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 862)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2023-06-20T22:38:46.862881+00:00',
        'data_type': 'chrome:cache:entry',
        'original_url': ('https://m.media-amazon.com/images/'
            'G/01/gno/sprites/nav-sprite-global-1x-reorg-privacy'
            '._CB587940754_.png')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains an interface for browser cookie plugins."""

import abc

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.lib import errors
from plaso.parsers import plugins


class BaseCookiePlugin(plugins.BasePlugin):
  """A browser cookie plugin for Plaso.

  This is a generic cookie parsing interface that can handle parsing
  cookies from all browsers.
  """
  NAME = 'cookie_plugin'
  DATA_FORMAT = 'Browser cookie data'

  # The name of the cookie value that this plugin is designed to parse.
  # This value is used to evaluate whether the plugin is the correct one
  # to parse the browser cookie.
  COOKIE_NAME = ''

  @abc.abstractmethod
  def _ParseCookieData(
      self, parser_mediator, cookie_data=None, url=None, **kwargs):
    """Extracts events from cookie data.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      cookie_data (Optional[bytes]): cookie data, as a byte sequence.
      url (Optional[str]): URL or path where the cookie was set.
    """

  def _ParseIntegerValue(self, string_value):
    """Parses an integer time.

    Args:
      string_value (str): string value.

    Returns:
      int: integer value or None if not available.
    """
    try:
      return int(string_value, 10)
    except (TypeError, ValueError):
      return None

  def _ParsePosixTime(self, string_value):
    """Parses a POSIX time.

    Args:
      string_value (str): string value.

    Returns:
      dfdatetime.PosixTime: date and time value or None if not available.
    """
    try:
      timestamp = int(string_value, 10)
    except (TypeError, ValueError):
      return None

    return dfdatetime_posix_time.PosixTime(timestamp=timestamp)

  def _ParsePosixTimeInMilliseconds(self, string_value):
    """Parses a POSIX time in milliseconds.

    Args:
      string_value (str): string value.

    Returns:
      dfdatetime.PosixTimeInMilliseconds: date and time value or None if not
          available.
    """
    try:
      timestamp = int(string_value, 10)
    except (TypeError, ValueError):
      return None

    return dfdatetime_posix_time.PosixTimeInMilliseconds(timestamp=timestamp)

  def _ParsePosixTimeIn100Nanoseconds(self, string_value):
    """Parses a POSIX time in 100 nanoseconds intervals.

    Args:
      string_value (str): string value.

    Returns:
      dfdatetime.PosixTimeInMicroseconds: date and time value or None if not
          available.
    """
    try:
      timestamp = int(string_value, 10)
      # TODO: fix that we're losing precision here.
      timestamp //= 10
    except (TypeError, ValueError):
      return None

    return dfdatetime_posix_time.PosixTimeInMicroseconds(timestamp=timestamp)

  # pylint: disable=arguments-differ
  def Process(self, parser_mediator, cookie_name, cookie_data, url, **kwargs):
    """Extracts events from cookie data.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      cookie_name (str): the name of the cookie value.
      cookie_data (bytes): the cookie data, as a byte sequence.
      url (str): the full URL or path where the cookie was set.

    Raises:
      errors.WrongPlugin: If the cookie name differs from the one
          supplied in COOKIE_NAME.
      ValueError: If cookie_name or cookie_data are not set.
    """
    if cookie_name is None or cookie_data is None:
      raise ValueError('Cookie name or data are not set.')

    if cookie_name != self.COOKIE_NAME:
      raise errors.WrongPlugin(
          'Not the correct cookie plugin for: {0:s} [{1:s}]'.format(
              cookie_name, self.NAME))

    # This will raise if unhandled keyword arguments are passed.
    super(BaseCookiePlugin, self).Process(parser_mediator)

    self._ParseCookieData(parser_mediator, cookie_data=cookie_data, url=url)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Script to plot CPU usage from profiling data.

This script requires the matplotlib and numpy Python modules.
"""

import argparse
import glob
import os
import sys

import numpy

from matplotlib import pyplot


def Main():
  """The main program function.

  Returns:
    bool: True if successful or False if not.
  """
  argument_parser = argparse.ArgumentParser(description=(
      'Plots CPU usage from profiling data.'))

  argument_parser.add_argument(
      '--output', dest='output_file', type=str, help=(
          'path of the output file to write the graph to instead of using '
          'interactive mode. The output format deduced from the extension '
          'of the filename.'))

  argument_parser.add_argument(
      '--process', dest='process', type=str, default='', help=(
          'comma separated list of names of processes to graph.'))

  argument_parser.add_argument(
      '--profiler', dest='profiler', type=str, default='processing', help=(
          'name of the CPU profiler to graph.'))

  argument_parser.add_argument(
      'profile_path', type=str, help=(
          'path to the directory containing the profiling data.'))

  options = argument_parser.parse_args()

  if not os.path.isdir(options.profile_path):
    print('No such directory: {0:s}'.format(options.profile_path))
    return False

  if options.profiler not in ('analyzers', 'parsers', 'processing'):
    print('Unsupported profiler: {0:s}'.format(options.profiler))
    return False

  processes = []
  if options.process:
    processes = options.process.split(',')

  if options.profiler == 'analyzers':
    name_prefix = 'analyzers'
    name_suffix = 'analyzers'

  elif options.profiler == 'parsers':
    name_prefix = 'cputime'
    name_suffix = 'parsers'

  elif options.profiler == 'processing':
    name_prefix = 'processing'
    name_suffix = 'processing'

  names = ['time', 'name', 'cpu']

  glob_pattern = '{0:s}-*-{1:s}.csv.gz'.format(name_prefix, name_suffix)
  glob_expression = os.path.join(options.profile_path, glob_pattern)
  for csv_file_name in glob.glob(glob_expression):
    process_name = os.path.basename(csv_file_name)
    process_name_prefix = '{0:s}-'.format(name_prefix)
    process_name_suffix = '-{0:s}.csv.gz'.format(name_suffix)
    process_name = process_name.replace(process_name_prefix, '').replace(
        process_name_suffix, '')
    if processes and process_name not in processes:
      continue

    data = numpy.genfromtxt(
        csv_file_name, delimiter='\t', dtype=None, encoding='utf-8',
        names=names, skip_header=1)

    if data.size > 0:
      for name in numpy.unique(data['name']):
        # Ignore process_sources since it is a single sample that contains
        # the cumulative CPU time.
        if options.profiler == 'processing' and name == 'process_sources':
          continue

        data_by_name = numpy.extract(data['name'] == name, data)
        label = '-'.join([name, process_name])
        pyplot.plot(data_by_name['time'], data_by_name['cpu'], label=label)

  pyplot.title('CPU usage over time')

  pyplot.xlabel('Time')
  pyplot.xscale('linear')

  pyplot.ylabel('Used CPU')
  pyplot.yscale('linear')

  pyplot.legend()

  if options.output_file:
    pyplot.savefig(options.output_file)
  else:
    pyplot.show()

  return True


if __name__ == '__main__':
  if not Main():
    sys.exit(1)
  else:
    sys.exit(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Apple Spotlight store database parser."""

import os
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.parsers import spotlight_storedb

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class SpotlightStoreDatabaseParserTest(test_lib.ParserTestCase):
  """Tests for the Apple Spotlight store database parser."""

  def testParse(self):
    """Tests the Parse function."""
    test_file_path = os.path.join(
        shared_test_lib.TEST_DATA_PATH, 'spotlight.10.13.dmg')
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_MODI, parent=test_path_spec)
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GPT, location='/p1',
        parent=test_path_spec)

    test_file_path = (
        '/.Spotlight-V100/Store-V2/D980C3E8-1007-4F67-9911-9143A0B3427A/'
        '.store.db')
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_HFS, location=test_file_path,
        parent=test_path_spec)

    parser = spotlight_storedb.SpotlightStoreDatabaseParser()
    storage_writer = self._ParseFileByPathSpec(test_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2023-06-22T18:34:06.000000+00:00',
        'attribute_change_time': None,
        'content_creation_time': '2023-06-22T18:34:06.000000+00:00',
        'content_modification_time': '2023-06-22T18:34:06.000000+00:00',
        'content_type': 'public.data',
        'creation_time': '2023-06-22T18:34:06.000000+00:00',
        'data_type': 'spotlight:metadata_item',
        'downloaded_time': None,
        'file_name': 'LICENSE',
        'file_system_identifier': 20,
        'kind': 'Unknown document',
        'modification_time': '2023-06-22T18:34:06.000000+00:00',
        'parent_file_system_identifier': 2,
        'purchase_time': None,
        'snapshot_times': None,
        'update_time': '2023-06-22T18:34:08.287881+00:00',
        'used_times': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

  def testParseWithLZ4CompressedPage(self):
    """Tests the Parse function on a file with a LZ4 compressed page."""
    parser = spotlight_storedb.SpotlightStoreDatabaseParser()
    storage_writer = self._ParseFile(['859631-store.db'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1848)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': None,
        'attribute_change_time': None,
        'content_creation_time': None,
        'content_modification_time': None,
        'content_type': None,
        'creation_time': None,
        'data_type': 'spotlight:metadata_item',
        'downloaded_time': None,
        'file_name': None,
        'file_system_identifier': None,
        'kind': None,
        'modification_time': None,
        'purchase_time': None,
        'snapshot_times': None,
        'update_time': '2019-09-17T09:22:07.536585+00:00',
        'used_times': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testParseWithStreamsMap(self):
    """Tests the Parse function on a database that uses a streams map."""
    test_file_path = os.path.join(
        shared_test_lib.TEST_DATA_PATH, 'spotlight.12.dmg')
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_MODI, parent=test_path_spec)
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GPT, location='/p1',
        parent=test_path_spec)

    test_file_path = (
        '/.Spotlight-V100/Store-V2/B8A60235-5AE9-4A1A-9004-3F40B6FF4C28/'
        '.store.db')
    test_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_HFS, location=test_file_path,
        parent=test_path_spec)

    parser = spotlight_storedb.SpotlightStoreDatabaseParser()
    storage_writer = self._ParseFileByPathSpec(test_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'added_time': '2023-06-20T05:10:24.000000+00:00',
        'attribute_change_time': None,
        'content_creation_time': '2023-06-20T05:10:24.000000+00:00',
        'content_modification_time': '2023-06-20T05:10:24.000000+00:00',
        'content_type': 'public.data',
        'creation_time': '2023-06-20T05:10:24.000000+00:00',
        'data_type': 'spotlight:metadata_item',
        'downloaded_time': None,
        'file_name': 'LICENSE',
        'file_system_identifier': 18,
        'kind': 'Document',
        'modification_time': '2023-06-20T05:10:24.000000+00:00',
        'parent_file_system_identifier': 2,
        'purchase_time': None,
        'snapshot_times': None,
        'update_time': '2023-06-21T03:42:12.717812+00:00',
        'used_times': None}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the JSON-L parser plugin for Docker container config files."""

import unittest

from plaso.parsers.jsonl_plugins import docker_container_config

from tests.parsers.jsonl_plugins import test_lib


class DockerContainerLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Docker container config files."""

  def testProcess(self):
    """Tests the Process function."""
    container_identifier = (
        'e7d0b7ea5ccf08366e2b0c8afa2318674e8aefe802315378125d2bb83fe3110c')
    path_segments = [
        'docker', 'containers', container_identifier, 'config.json']

    plugin = docker_container_config.DockerContainerConfigurationJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(path_segments, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'container_identifier': container_identifier,
        'container_name': 'e7d0b7ea5ccf',
        'creation_time': '2016-01-07T16:49:08.507979+00:00',
        'data_type': 'docker:container:configuration',
        'start_time': '2016-01-07T16:49:08.674873+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the OLE Compound File summary and document summary plugins."""

import unittest

from plaso.parsers.olecf_plugins import summary

from tests.parsers.olecf_plugins import test_lib


class TestSummaryInformationOLECFPlugin(test_lib.OLECFPluginTestCase):
  """Tests for the OLECF summary information plugin."""

  def testProcess(self):
    """Tests the Process function on a Summary Information stream."""
    plugin = summary.SummaryInformationOLECFPlugin()
    storage_writer = self._ParseOLECFFileWithPlugin(['Document.doc'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application': 'Microsoft Office Word',
        'author': 'DAVID NIDES',
        'creation_time': '2012-12-10T18:38:00.0000000+00:00',
        'data_type': 'olecf:summary_info',
        'edit_duration': None,
        'item_creation_time': None,
        'item_modification_time': '2013-05-16T02:29:49.7950000+00:00',
        'last_printed_time': None,
        'last_save_time': '2013-05-16T02:29:00.0000000+00:00',
        'last_saved_by': 'Nides',
        'number_of_characters': 18,
        'number_of_pages': 1,
        'number_of_words': 3,
        'revision_number': '4',
        'security_flags': 0,
        'template': 'Normal.dotm',
        'title': 'Table of Context'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class TestDocumentSummaryInformationOLECFPlugin(test_lib.OLECFPluginTestCase):
  """Tests for the OLECF document summary information plugin."""

  def testProcess(self):
    """Tests the Process function on a Document Summary Information stream."""
    plugin = summary.DocumentSummaryInformationOLECFPlugin()
    storage_writer = self._ParseOLECFFileWithPlugin(['Document.doc'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application_version': '14.0',
        'company': 'KPMG',
        'data_type': 'olecf:document_summary_info',
        'item_creation_time': None,
        'item_modification_time': '2013-05-16T02:29:49.7950000+00:00',
        'links_up_to_date': False,
        'number_of_lines': 1,
        'number_of_paragraphs': 1,
        'shared_document': False}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows Registry plugin related functions and classes for testing."""

from dfvfs.path import fake_path_spec

from dfwinreg import fake as dfwinreg_fake
from dfwinreg import regf as dfwinreg_regf
from dfwinreg import registry as dfwinreg_registry

from plaso.containers import events
from plaso.parsers import mediator as parsers_mediator

from tests.parsers import test_lib


class TestFileEntry(object):
  """File entry object for testing purposes.

  Attributes:
    name (str): name of the file entry.
    path_spec (dfvfs.PathSpec): path specification of the file entry.
  """

  def __init__(self, name):
    """Initializes a file entry.

    Args:
      name (str): the file entry name.
    """
    super(TestFileEntry, self).__init__()
    self.name = name
    self.path_spec = fake_path_spec.FakePathSpec(location=name)


class RegistryPluginTestCase(test_lib.ParserTestCase):
  """The unit test case for a Windows Registry plugin."""

  # pylint: disable=protected-access

  def _AssertFiltersOnKeyPath(self, plugin, key_path):
    """Asserts if the key path matches one of the plugin filters.

    Args:
      plugin (WindowsRegistryPlugin): Windows Registry plugin.
      key_path (str): Windows Registry key path.
    """
    _, _, key_name = key_path.rpartition('\\')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(key_name, key_path=key_path)

    result = self._CheckFiltersOnKeyPath(plugin, registry_key)
    self.assertTrue(result)

  def _AssertNotFiltersOnKeyPath(self, plugin, key_path):
    """Asserts if the key path does not match one of the plugin filters.

    Args:
      plugin (WindowsRegistryPlugin): Windows Registry plugin.
      key_path (str): Windows Registry key path.
    """
    _, _, key_name = key_path.rpartition('\\')
    registry_key = dfwinreg_fake.FakeWinRegistryKey(key_name, key_path=key_path)

    result = self._CheckFiltersOnKeyPath(plugin, registry_key)
    self.assertFalse(result)

  def _CheckFiltersOnKeyPath(self, plugin, registry_key):
    """Checks if the key path matches one of the plugin filters.

    Args:
      plugin (WindowsRegistryPlugin): Windows Registry plugin.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.

    Returns:
      bool: True if the key path matches one of the plugin filters,
          False otherwise.
    """
    result = False
    for path_filter in plugin.FILTERS:
      if path_filter.Match(registry_key):
        result = True

    return result

  def _GetWinRegistryFromFileEntry(self, file_entry):
    """Retrieves a Windows Registry from a file entry.

    Args:
      file_entry (dfvfs.FileEntry): file entry that references a test file.

    Returns:
      dfwinreg.WinRegistry: Windows Registry or None.
    """
    file_object = file_entry.GetFileObject()
    if not file_object:
      return None

    registry_file = dfwinreg_regf.REGFWinRegistryFile(
        ascii_codepage='cp1252', emulate_virtual_keys=False)
    registry_file.Open(file_object)

    win_registry = dfwinreg_registry.WinRegistry()
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    return win_registry

  def _ParseKeyWithPlugin(self, registry_key, plugin, file_entry=None):
    """Parses a key within a Windows Registry file using the plugin.

    Args:
      registry_key (dfwinreg.WinRegistryKey): Windows Registry Key.
      plugin (WindowsRegistryPlugin): Windows Registry plugin.
      file_entry (Optional[dfvfs.FileEntry]): file entry.

    Returns:
      FakeStorageWriter: storage writer.

    Raises:
      SkipTest: if the path inside the test data directory does not exist and
          the test should be skipped.
    """
    self.assertIsNotNone(registry_key)

    parser_mediator = parsers_mediator.ParserMediator()

    storage_writer = self._CreateStorageWriter()
    parser_mediator.SetStorageWriter(storage_writer)

    parser_mediator.SetFileEntry(file_entry)

    if file_entry:
      event_data_stream = events.EventDataStream()
      event_data_stream.path_spec = file_entry.path_spec

      parser_mediator.ProduceEventDataStream(event_data_stream)

    # AppendToParserChain needs to be run after SetFileEntry.
    parser_mediator.AppendToParserChain('winreg')

    plugin.UpdateChainAndProcess(parser_mediator, registry_key)

    return storage_writer



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the unique domains visited analysis plugin."""

import unittest

from plaso.analysis import unique_domains_visited
from plaso.containers import reports
from plaso.lib import definitions

from tests.analysis import test_lib


class UniqueDomainsPluginTest(test_lib.AnalysisPluginTestCase):
  """Tests for the unique domains analysis plugin."""

  _TEST_EVENTS = [
      {'_parser_chain': 'sqlite/chrome_history',
       'data_type': 'chrome:history:file_downloaded',
       'domain': 'firstevent.com',
       'path': '/1/index.html',
       'timestamp': '2015-01-01 01:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'url': 'https://firstevent.com/1/index.html'},
      {'_parser_chain': 'sqlite/firefox_history',
       'data_type': 'firefox:places:page_visited',
       'domain': 'secondevent.net',
       'path': '/2/index.html',
       'timestamp': '2015-02-02 02:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'url': 'https://secondevent.net/2/index.html'},
      {'_parser_chain': 'msiecf',
       'data_type': 'msiecf:redirected',
       'domain': 'thirdevent.org',
       'path': '/3/index.html',
       'timestamp': '2015-03-03 03:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'url': 'https://thirdevent.org/3/index.html'},
      {'_parser_chain': 'safari_history',
       'data_type': 'safari:history:visit',
       'domain': 'fourthevent.co',
       'path': '/4/index.html',
       'timestamp': '2015-04-04 04:00:00',
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN,
       'url': 'https://fourthevent.co/4/index.html'}]

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    plugin = unique_domains_visited.UniqueDomainsVisitedPlugin()
    storage_writer = self._AnalyzeEvents(self._TEST_EVENTS, plugin)

    number_of_reports = storage_writer.GetNumberOfAttributeContainers(
        'analysis_report')
    self.assertEqual(number_of_reports, 1)

    analysis_report = storage_writer.GetAttributeContainerByIndex(
        reports.AnalysisReport.CONTAINER_TYPE, 0)

    self.assertEqual(analysis_report.analysis_counter['firstevent.com'], 1)
    self.assertEqual(analysis_report.analysis_counter['secondevent.net'], 1)
    self.assertEqual(analysis_report.analysis_counter['thirdevent.org'], 1)
    self.assertEqual(analysis_report.analysis_counter['fourthevent.co'], 1)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Tests for the JSON-L parser plugin for Microsoft 365 audit log files."""

import unittest

from plaso.parsers.jsonl_plugins import microsoft365_audit_log

from tests.parsers.jsonl_plugins import test_lib


class Microsoft365AuditLogJSONLPluginTest(test_lib.JSONLPluginTestCase):
  """Tests for the JSON-L parser plugin for Microsoft 365 audit log files."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = microsoft365_audit_log.Microsoft365AuditLogJSONLPlugin()
    storage_writer = self._ParseJSONLFileWithPlugin(
        ['microsoft_audit_log.jsonl'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'audit_record_identifier': 'c9d2d808-0efe-48cb-eaec-08da3028eb80',
        'operation_name': 'Install-DefaultSharingPolicy',
        'organization_identifier': '5a0f38c6-710b-4503-92c0-3a9f6e00f726',
        'record_type': 1,
        'recorded_time': '2022-05-07T12:55:53+00:00',
        'result_status': 'True',
        'user_key': 'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)',
        'user_type': 3,
        'workload': 'Exchange',
        'object_identifier': (
            'sst5f.onmicrosoft.com\\952cae95-808b-4aa7-b783-a9151be9a05a'),
        'user_identifier': (
            'NT AUTHORITY\\SYSTEM (Microsoft.Exchange.Servicehost)')}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Analysis plugin to look up file hashes in nsrlsvr and tag events."""

import socket

from plaso.analysis import hash_tagging
from plaso.analysis import logger
from plaso.analysis import manager


class NsrlsvrAnalysisPlugin(hash_tagging.HashTaggingAnalysisPlugin):
  """Analysis plugin for looking up hashes in nsrlsvr."""

  # The NSRL contains files of all different types, and can handle a high load
  # so look up all files.
  DATA_TYPES = frozenset(['fs:stat', 'fs:stat:ntfs'])

  NAME = 'nsrlsvr'

  SUPPORTED_HASHES = frozenset(['md5', 'sha1'])

  DEFAULT_LABEL = 'nsrl_present'

  _RECEIVE_BUFFER_SIZE = 4096

  _SOCKET_TIMEOUT = 3

  def __init__(self):
    """Initializes an nsrlsvr analysis plugin."""
    super(NsrlsvrAnalysisPlugin, self).__init__()
    self._host = None
    self._label = self.DEFAULT_LABEL
    self._port = None

  def _Analyze(self, hashes):
    """Looks up file hashes in nsrlsvr.

    Args:
      hashes (list[str]): hash values to look up.

    Returns:
      list[HashAnalysis]: analysis results, or an empty list on error.
    """
    logger.debug('Opening connection to {0:s}:{1:d}'.format(
        self._host, self._port))

    nsrl_socket = self._GetSocket()
    if not nsrl_socket:
      self.SignalAbort()
      return []

    hash_analyses = []
    for digest in hashes:
      response = self._QueryHash(nsrl_socket, digest)
      if response is not None:
        hash_analysis = hash_tagging.HashAnalysis(digest, response)
        hash_analyses.append(hash_analysis)

    nsrl_socket.close()

    logger.debug('Closed connection to {0:s}:{1:d}'.format(
        self._host, self._port))

    return hash_analyses

  def _GenerateLabels(self, hash_information):
    """Generates a list of strings that will be used in the event tag.

    Args:
      hash_information (bool): response from the hash tagging that indicates
          that the file hash was present or not.

    Returns:
      list[str]: list of labels to apply to event.
    """
    if hash_information:
      return [self._label]

    # TODO: Renable when tagging is removed from the analysis report.
    # return ['nsrl_not_present']
    return []

  def _GetSocket(self):
    """Establishes a connection to an nsrlsvr instance.

    Returns:
      socket._socketobject: socket connected to an nsrlsvr instance or None if
          a connection cannot be established.
    """
    try:
      connected_socket = socket.create_connection(
          (self._host, self._port), self._SOCKET_TIMEOUT)

    except socket.error as exception:
      connected_socket = None
      logger.error('Unable to connect to nsrlsvr with error: {0!s}.'.format(
          exception))

    return connected_socket

  def _QueryHash(self, nsrl_socket, digest):
    """Queries nsrlsvr for a specific hash.

    Args:
      nsrl_socket (socket._socketobject): socket of connection to nsrlsvr.
      digest (str): hash to look up.

    Returns:
      bool: True if the hash was found, False if not or None on error.
    """
    try:
      query = 'QUERY {0:s}\n'.format(digest).encode('ascii')
    except UnicodeDecodeError:
      logger.error('Unable to encode digest: {0!s} to ASCII.'.format(digest))
      return False

    response = None

    try:
      nsrl_socket.sendall(query)
      response = nsrl_socket.recv(self._RECEIVE_BUFFER_SIZE)

    except socket.error as exception:
      logger.error('Unable to query nsrlsvr with error: {0!s}.'.format(
          exception))

    if not response:
      return False

    # Strip end-of-line characters since they can differ per platform on which
    # nsrlsvr is running.
    response = response.strip()
    # nsrlsvr returns "OK 1" if the has was found or "OK 0" if not.
    return response == b'OK 1'

  def SetLabel(self, label):
    """Sets the tagging label.

    Args:
      label (str): label to apply to events extracted from files that are
          present in nsrlsvr.
    """
    self._label = label

  def SetHost(self, host):
    """Sets the address or hostname of the server running nsrlsvr.

    Args:
      host (str): IP address or hostname to query.
    """
    self._host = host

  def SetPort(self, port):
    """Sets the port where nsrlsvr is listening.

    Args:
      port (int): port to query.
    """
    self._port = port

  def TestConnection(self):
    """Tests the connection to nsrlsvr.

    Checks if a connection can be set up and queries the server for the
    MD5 of an empty file and expects a response. The value of the response
    is not checked.

    Returns:
      bool: True if nsrlsvr instance is reachable.
    """
    response = None
    nsrl_socket = self._GetSocket()
    if nsrl_socket:
      response = self._QueryHash(
          nsrl_socket, 'd41d8cd98f00b204e9800998ecf8427e')
      nsrl_socket.close()

    return response is not None


manager.AnalysisPluginManager.RegisterPlugin(NsrlsvrAnalysisPlugin)
